@@ -1,6 +1,4 @@
 /*
- *  arch-tag: Header with definitions for Rhythmbox icon loading
- *
  *  Copyright (C) 2002 Jorn Baayen
  *  Copyright (C) 2004 Colin Walters <walters@verbum.org>
  *
@@ -9,6 +7,14 @@
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -25,8 +31,8 @@
 
 G_BEGIN_DECLS
 
-extern const char RB_STOCK_TRAY_ICON_PLAYING[];
-extern const char RB_STOCK_TRAY_ICON_NOT_PLAYING[];
+extern const char RB_APP_ICON[];
+extern const char RB_STOCK_TRAY_ICON[];
 extern const char RB_STOCK_SET_STAR[];
 extern const char RB_STOCK_UNSET_STAR[];
 extern const char RB_STOCK_NO_STAR[];
