@@ -40,8 +40,8 @@
 #include <glib-object.h>
 #include <glib/gstdio.h>
 #include <gdk/gdkx.h>
-#include <brasero/brasero-media.h>
-#include <brasero/brasero-medium-monitor.h>
+#include <brasero-media.h>
+#include <brasero-medium-monitor.h>
 
 #include <libxml/xmlerror.h>
 #include <libxml/xmlwriter.h>
@@ -50,7 +50,7 @@
 #include <libxml/uri.h>
 #include <libxml/xmlsave.h>
 
-#include "rb-plugin.h"
+#include "rb-plugin-macros.h"
 #include "rb-debug.h"
 #include "rb-shell.h"
 #include "rb-source.h"
@@ -67,28 +67,25 @@
 
 typedef struct
 {
-	RBPlugin        parent;
+	PeasExtensionBase parent;
 
-	RBShell        *shell;
 	GtkActionGroup *action_group;
 	guint           ui_merge_id;
 
-	RBSource       *selected_source;
+	RBDisplayPage  *selected_page;
 	guint           enabled : 1;
 } RBDiscRecorderPlugin;
 
 typedef struct
 {
-	RBPluginClass parent_class;
+	PeasExtensionBaseClass parent_class;
 } RBDiscRecorderPluginClass;
 
-G_MODULE_EXPORT GType register_rb_plugin (GTypeModule *module);
-GType	rb_disc_recorder_plugin_get_type		(void) G_GNUC_CONST;
+G_MODULE_EXPORT void peas_register_types (PeasObjectModule *module);
+
+RB_DEFINE_PLUGIN(RB_TYPE_DISC_RECORDER_PLUGIN, RBDiscRecorderPlugin, rb_disc_recorder_plugin,)
 
 static void rb_disc_recorder_plugin_init (RBDiscRecorderPlugin *plugin);
-static void rb_disc_recorder_plugin_finalize (GObject *object);
-static void impl_activate (RBPlugin *plugin, RBShell *shell);
-static void impl_deactivate (RBPlugin *plugin, RBShell *shell);
 static void cmd_burn_source (GtkAction          *action,
 			     RBDiscRecorderPlugin *pi);
 static void cmd_duplicate_cd (GtkAction          *action,
@@ -103,8 +100,6 @@ static GtkActionEntry rb_disc_recorder_plugin_actions [] = {
 	  G_CALLBACK (cmd_duplicate_cd) },
 };
 
-RB_PLUGIN_REGISTER(RBDiscRecorderPlugin, rb_disc_recorder_plugin)
-
 #define RB_RECORDER_ERROR rb_disc_recorder_error_quark ()
 
 GQuark rb_disc_recorder_error_quark (void);
@@ -126,44 +121,24 @@ typedef enum
 	RB_RECORDER_ERROR_GENERAL
 } RBRecorderError;
 
-static void
-rb_disc_recorder_plugin_class_init (RBDiscRecorderPluginClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	RBPluginClass *plugin_class = RB_PLUGIN_CLASS (klass);
-
-	object_class->finalize = rb_disc_recorder_plugin_finalize;
-
-	plugin_class->activate = impl_activate;
-	plugin_class->deactivate = impl_deactivate;
-}
-
 static void
 rb_disc_recorder_plugin_init (RBDiscRecorderPlugin *pi)
 {
 	rb_debug ("RBDiscRecorderPlugin initialized");
 }
 
-static void
-rb_disc_recorder_plugin_finalize (GObject *object)
-{
-	rb_debug ("RBDiscRecorderPlugin finalized");
-
-	G_OBJECT_CLASS (rb_disc_recorder_plugin_parent_class)->finalize (object);
-}
-
 static gboolean
 rb_disc_recorder_plugin_start_burning (RBDiscRecorderPlugin *pi,
 					  const char *path,
 					  gboolean copy)
 {
 	GtkWidget *main_window;
-	GdkScreen *screen;
 	GdkWindow *window;
 	GPtrArray *array;
 	char **args, *xid_str;
 	GError *error = NULL;
 	gboolean ret;
+	RBShell *shell;
 	
 	array = g_ptr_array_new ();
 	g_ptr_array_add (array, "brasero");
@@ -173,12 +148,14 @@ rb_disc_recorder_plugin_start_burning (RBDiscRecorderPlugin *pi,
 		g_ptr_array_add (array, "-r");
 	g_ptr_array_add (array, (gpointer) path);
 
-	main_window = gtk_widget_get_toplevel (GTK_WIDGET (pi->selected_source));
-	screen = gtk_widget_get_screen (main_window);
+	g_object_get (pi, "object", &shell, NULL);
+	g_object_get (shell, "window", &main_window, NULL);
+	g_object_unref (shell);
+
 	window = gtk_widget_get_window (main_window);
 	if (window) {
 		int xid;
-		xid = gdk_x11_drawable_get_xid (GDK_DRAWABLE (window));
+		xid = gdk_x11_window_get_xid (window);
 		xid_str = g_strdup_printf ("%d", xid);
 		g_ptr_array_add (array, "-x");
 		g_ptr_array_add (array, xid_str);
@@ -190,7 +167,7 @@ rb_disc_recorder_plugin_start_burning (RBDiscRecorderPlugin *pi,
 	args = (char **) g_ptr_array_free (array, FALSE);
 
 	ret = TRUE;
-	if (!gdk_spawn_on_screen (screen, NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error)) {
+	if (!g_spawn_async (NULL, args, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error)) {
 		if (copy != FALSE) {
 			rb_error_dialog (GTK_WINDOW (main_window),
 					 _("Rhythmbox could not duplicate the disc"),
@@ -494,8 +471,8 @@ static void
 cmd_burn_source (GtkAction            *action,
 		 RBDiscRecorderPlugin *pi)
 {
-	if (pi->selected_source != NULL) {
-		source_burn (pi, pi->selected_source);
+	if (pi->selected_page != NULL && RB_IS_SOURCE (pi->selected_page)) {
+		source_burn (pi, RB_SOURCE (pi->selected_page));
 	}
 }
 
@@ -506,10 +483,10 @@ cmd_duplicate_cd (GtkAction         	*action,
 	gchar *device;
 	GVolume *volume;
 
-	if (!pi->selected_source)
+	if (pi->selected_page == NULL || RB_IS_SOURCE (pi->selected_page) == FALSE)
 		return;
 
-	g_object_get (pi->selected_source, "volume", &volume, NULL);
+	g_object_get (pi->selected_page, "volume", &volume, NULL);
 	if (G_IS_VOLUME (volume))
 		device = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	else
@@ -591,25 +568,29 @@ update_source (RBDiscRecorderPlugin *pi,
 {
 	GtkAction *burn_action, *copy_action;
 	gboolean   playlist_active, is_audiocd_active;
-	RBSource  *selected_source;
-	const char *source_type;
+	RBDisplayPage *selected_page;
+	const char *page_type;
 
-	if (pi->selected_source != NULL) {
+	if (pi->selected_page != NULL && RB_IS_SOURCE (pi->selected_page)) {
 		RhythmDBQueryModel *model;
 
-		g_object_get (pi->selected_source, "query-model", &model, NULL);
+		g_object_get (pi->selected_page, "query-model", &model, NULL);
 		g_signal_handlers_disconnect_by_func (model, playlist_row_inserted_cb, pi);
 		g_signal_handlers_disconnect_by_func (model, playlist_entries_changed, pi);
 		g_object_unref (model);
 	}
 
-	g_object_get (shell, "selected-source", &selected_source, NULL);
+	g_object_get (shell, "selected-page", &selected_page, NULL);
 
 	/* for now restrict to playlist sources */
-	playlist_active = RB_IS_PLAYLIST_SOURCE (selected_source);
+	playlist_active = RB_IS_PLAYLIST_SOURCE (selected_page);
 
-	source_type = G_OBJECT_TYPE_NAME (selected_source);
-	is_audiocd_active = g_str_equal (source_type, "RBAudioCdSource");
+	if (selected_page != NULL) {
+		page_type = G_OBJECT_TYPE_NAME (selected_page);
+		is_audiocd_active = g_str_equal (page_type, "RBAudioCdSource");
+	} else {
+		is_audiocd_active = FALSE;
+	}
 
 	burn_action = gtk_action_group_get_action (pi->action_group,
 						   "MusicPlaylistBurnToDiscPlaylist");
@@ -619,7 +600,7 @@ update_source (RBDiscRecorderPlugin *pi,
 	if (pi->enabled && playlist_active && rb_disc_recorder_has_burner (pi)) {
 		RhythmDBQueryModel *model;
 
-		g_object_get (selected_source, "query-model", &model, NULL);
+		g_object_get (selected_page, "query-model", &model, NULL);
 		/* monitor for changes, to enable/disable the burn menu item */
 		g_signal_connect_object (G_OBJECT (model),
 					 "row_inserted",
@@ -643,19 +624,18 @@ update_source (RBDiscRecorderPlugin *pi,
 		gtk_action_set_visible (copy_action, FALSE);
 	}
 
-	if (pi->selected_source != NULL) {
-		g_object_unref (pi->selected_source);
+	if (pi->selected_page != NULL) {
+		g_object_unref (pi->selected_page);
 	}
-	pi->selected_source = selected_source;
+	pi->selected_page = selected_page;
 }
 
 static void
-shell_selected_source_notify_cb (RBShell            *shell,
-				 GParamSpec         *param,
-				 RBDiscRecorderPlugin *pi)
+shell_selected_page_notify_cb (RBShell            *shell,
+			       GParamSpec         *param,
+			       RBDiscRecorderPlugin *pi)
 {
-	rb_debug ("RBDiscRecorderPlugin selected source changed");
-
+	rb_debug ("RBDiscRecorderPlugin selected page changed");
 	update_source (pi, shell);
 }
 
@@ -674,29 +654,29 @@ static struct ui_paths {
 };
 
 static void
-impl_activate (RBPlugin *plugin,
-	       RBShell  *shell)
+impl_activate (PeasActivatable *plugin)
 {
 	RBDiscRecorderPlugin *pi = RB_DISC_RECORDER_PLUGIN (plugin);
 	GtkUIManager         *uimanager = NULL;
 	GtkAction            *action;
+	RBShell              *shell;
 	int                   i;
 
+	g_object_get (pi, "object", &shell, NULL);
+
 	pi->enabled = TRUE;
 
 	rb_debug ("RBDiscRecorderPlugin activating");
 
 	brasero_media_library_start ();
 
-	pi->shell = shell;
-
 	g_object_get (shell,
 		      "ui-manager", &uimanager,
 		      NULL);
 
 	g_signal_connect_object (G_OBJECT (shell),
-				 "notify::selected-source",
-				 G_CALLBACK (shell_selected_source_notify_cb),
+				 "notify::selected-page",
+				 G_CALLBACK (shell_selected_page_notify_cb),
 				 pi, 0);
 
 	/* add UI */
@@ -740,14 +720,18 @@ impl_activate (RBPlugin *plugin,
 	g_object_set (action, "short-label", _("Copy CD"), NULL);
 
 	update_source (pi, shell);
+
+	g_object_unref (shell);
 }
 
 static void
-impl_deactivate	(RBPlugin *plugin,
-		 RBShell  *shell)
+impl_deactivate	(PeasActivatable *plugin)
 {
 	RBDiscRecorderPlugin *pi = RB_DISC_RECORDER_PLUGIN (plugin);
-	GtkUIManager       *uimanager = NULL;
+	GtkUIManager         *uimanager = NULL;
+	RBShell              *shell;
+
+	g_object_get (pi, "object", &shell, NULL);
 
 	pi->enabled = FALSE;
 
@@ -755,12 +739,12 @@ impl_deactivate	(RBPlugin *plugin,
 
 	update_source (pi, shell);
 
-	if (pi->selected_source) {
-		g_object_unref (pi->selected_source);
-		pi->selected_source = NULL;
+	if (pi->selected_page) {
+		g_object_unref (pi->selected_page);
+		pi->selected_page = NULL;
 	}
 
-	g_signal_handlers_disconnect_by_func (shell, shell_selected_source_notify_cb, pi);
+	g_signal_handlers_disconnect_by_func (shell, shell_selected_page_notify_cb, pi);
 
 	g_object_get (shell,
 		      "ui-manager", &uimanager,
@@ -772,5 +756,15 @@ impl_deactivate	(RBPlugin *plugin,
 	g_object_unref (uimanager);
 
 	/* NOTE: don't deactivate libbrasero-media as it could be in use somewhere else */
+
+	g_object_unref (shell);
 }
 
+G_MODULE_EXPORT void
+peas_register_types (PeasObjectModule *module)
+{
+	rb_disc_recorder_plugin_register_type (G_TYPE_MODULE (module));
+	peas_object_module_register_extension_type (module,
+						    PEAS_TYPE_ACTIVATABLE,
+						    RB_TYPE_DISC_RECORDER_PLUGIN);
+}
