@@ -41,14 +41,12 @@
 
 #include "rhythmdb.h"
 #include "rb-shell.h"
-#include "eel-gconf-extensions.h"
 #include "rb-daap-source.h"
 #include "rb-stock-icons.h"
 #include "rb-debug.h"
 #include "rb-util.h"
 #include "rb-file-helpers.h"
 #include "rb-dialog.h"
-#include "rb-preferences.h"
 #include "rb-daap-src.h"
 #include "rb-daap-record-factory.h"
 #include "rb-rhythmdb-dmap-db-adapter.h"
@@ -61,6 +59,9 @@
 
 #include <libdmapsharing/dmap.h>
 
+typedef struct _RhythmDBEntryType RBDAAPEntryType;
+typedef struct _RhythmDBEntryTypeClass RBDAAPEntryTypeClass;
+
 static void rb_daap_source_dispose (GObject *object);
 static void rb_daap_source_set_property  (GObject *object,
 					  guint prop_id,
@@ -75,12 +76,9 @@ static void rb_daap_source_selected (RBDisplayPage *page);
 static gboolean rb_daap_source_show_popup (RBDisplayPage *page);
 static void rb_daap_source_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress);
 
-static char * rb_daap_source_get_browser_key (RBSource *source);
-static char * rb_daap_source_get_paned_key (RBBrowserSource *source);
-
-#define CONF_STATE_SORTING CONF_PREFIX "/state/daap/sorting"
-#define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/daap/paned_position"
-#define CONF_STATE_SHOW_BROWSER CONF_PREFIX "/state/daap/show_browser"
+static void rb_daap_entry_type_class_init (RBDAAPEntryTypeClass *klass);
+static void rb_daap_entry_type_init (RBDAAPEntryType *etype);
+GType rb_daap_entry_type_get_type (void);
 
 struct RBDAAPSourcePrivate
 {
@@ -112,7 +110,9 @@ enum {
 	PROP_PASSWORD_PROTECTED
 };
 
-G_DEFINE_TYPE (RBDAAPSource, rb_daap_source, RB_TYPE_BROWSER_SOURCE)
+G_DEFINE_DYNAMIC_TYPE (RBDAAPSource, rb_daap_source, RB_TYPE_BROWSER_SOURCE);
+
+G_DEFINE_DYNAMIC_TYPE (RBDAAPEntryType, rb_daap_entry_type, RHYTHMDB_TYPE_ENTRY_TYPE);
 
 static char *
 rb_daap_entry_type_get_playback_uri (RhythmDBEntryType *etype, RhythmDBEntry *entry)
@@ -127,6 +127,23 @@ rb_daap_entry_type_get_playback_uri (RhythmDBEntryType *etype, RhythmDBEntry *en
 	return g_strdup (location);
 }
 
+static void
+rb_daap_entry_type_class_init (RBDAAPEntryTypeClass *klass)
+{
+	RhythmDBEntryTypeClass *etype_class = RHYTHMDB_ENTRY_TYPE_CLASS (klass);
+	etype_class->get_playback_uri = rb_daap_entry_type_get_playback_uri;
+}
+
+static void
+rb_daap_entry_type_class_finalize (RBDAAPEntryTypeClass *klass)
+{
+}
+
+static void
+rb_daap_entry_type_init (RBDAAPEntryType *etype)
+{
+}
+
 static void
 rb_daap_source_dispose (GObject *object)
 {
@@ -169,9 +186,7 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
-	source_class->impl_get_browser_key = rb_daap_source_get_browser_key;
 
-	browser_source_class->impl_get_paned_key = rb_daap_source_get_paned_key;
 	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 
 	g_object_class_install_property (object_class,
@@ -210,6 +225,11 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	g_type_class_add_private (klass, sizeof (RBDAAPSourcePrivate));
 }
 
+static void
+rb_daap_source_class_finalize (RBDAAPSourceClass *klass)
+{
+}
+
 static void
 rb_daap_source_init (RBDAAPSource *source)
 {
@@ -280,7 +300,7 @@ rb_daap_source_get_property (GObject *object,
 
 RBSource *
 rb_daap_source_new (RBShell *shell,
-		    RBPlugin *plugin,
+		    GObject *plugin,
 		    const char *service_name,
 		    const char *name,
 		    const char *host,
@@ -292,22 +312,24 @@ rb_daap_source_new (RBShell *shell,
 	GdkPixbuf *icon;
 	RhythmDB *db;
 	char *entry_type_name;
+	GSettings *settings;
 
 	g_object_get (shell, "db", &db, NULL);
 	entry_type_name = g_strdup_printf ("daap:%s:%s:%s", service_name, name, host);
 
-	entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+	entry_type = g_object_new (rb_daap_entry_type_get_type (),
 				   "db", db,
 				   "name", entry_type_name,
 				   "save-to-disk", FALSE,
 				   "category", RHYTHMDB_ENTRY_NORMAL,
 				   NULL);
-	entry_type->get_playback_uri = rb_daap_entry_type_get_playback_uri;
 	rhythmdb_register_entry_type (db, entry_type);
 	g_object_unref (db);
 	g_free (entry_type_name);
 
 	icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin), password_protected, FALSE);
+
+	settings = g_settings_new ("org.gnome.rhythmbox.plugins.daap");
 	source = RB_SOURCE (g_object_new (RB_TYPE_DAAP_SOURCE,
 					  "service-name", service_name,
 					  "name", name,
@@ -317,10 +339,11 @@ rb_daap_source_new (RBShell *shell,
 					  "pixbuf", icon,
 					  "shell", shell,
 					  "visibility", TRUE,
-					  "sorting-key", CONF_STATE_SORTING,
 					  "password-protected", password_protected,
-					  "plugin", RB_PLUGIN (plugin),
+					  "plugin", G_OBJECT (plugin),
+					  "settings", g_settings_get_child (settings, "source"),
 					  NULL));
+	g_object_unref (settings);
 
 	if (icon != NULL) {
 		g_object_unref (icon);
@@ -406,16 +429,24 @@ ask_password (RBDAAPSource *source, const char *name, const char *keyring)
 #endif
 	}
 
+	if (! password) {
+		rb_daap_source_disconnect (source);
+	}
+
 	return password;
 }
 
-static char *
-connection_auth_cb (DMAPConnection   *connection,
-		    const char       *name,
-		    RBDAAPSource     *source)
+static void
+connection_auth_cb (DMAPConnection *connection,
+                    const char     *name,
+                    SoupSession    *session,
+                    SoupMessage    *msg,
+                    SoupAuth       *auth,
+                    gboolean        retrying,
+		    RBDAAPSource   *source)
 {
-#ifdef WITH_GNOME_KEYRING
 	gchar *password = NULL;
+#ifdef WITH_GNOME_KEYRING
 	GnomeKeyringResult keyringret;
 	gchar *keyring;
 	GList *list;
@@ -450,11 +481,10 @@ connection_auth_cb (DMAPConnection   *connection,
 	if (list)
 		gnome_keyring_network_password_list_free (list); */
 	g_free (keyring);
-	return password;
 #else
-	return ask_password (source, name, NULL);
+	password = ask_password (source, name, NULL);
 #endif
-
+	dmap_connection_authenticate_message (connection, session, msg, auth, password);
 }
 
 static void
@@ -465,13 +495,12 @@ connection_connecting_cb (DMAPConnection       *connection,
 {
 	GdkPixbuf *icon;
 	gboolean   is_connected;
-	RBPlugin  *plugin;
+	GObject *plugin;
 
 	rb_debug ("DAAP connection status: %d/%f", state, progress);
 
 	switch (state) {
 	case DMAP_GET_INFO:
-	case DMAP_GET_PASSWORD:
 	case DMAP_LOGIN:
 		source->priv->connection_status = _("Connecting to music share");
 		break;
@@ -490,7 +519,7 @@ connection_connecting_cb (DMAPConnection       *connection,
 
 	source->priv->connection_progress = progress;
 
-	_rb_display_page_notify_status_changed (RB_DISPLAY_PAGE (source));
+	rb_display_page_notify_status_changed (RB_DISPLAY_PAGE (source));
 
 	is_connected = dmap_connection_is_connected (DMAP_CONNECTION (connection));
 
@@ -500,7 +529,7 @@ connection_connecting_cb (DMAPConnection       *connection,
 	icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin),
 					source->priv->password_protected,
 					is_connected);
-	g_object_set (source, "icon", icon, NULL);
+	g_object_set (source, "pixbuf", icon, NULL);
 	if (icon != NULL) {
 		g_object_unref (icon);
 	}
@@ -513,16 +542,14 @@ connection_disconnected_cb (DMAPConnection   *connection,
 			    RBDAAPSource     *source)
 {
 	GdkPixbuf *icon;
-	RBPlugin  *plugin;
-	gboolean   daap_shutdown;
+	GObject   *plugin;
 
 	rb_debug ("DAAP connection disconnected");
 
 	g_object_get (source, "plugin", &plugin, NULL);
 	g_assert (plugin != NULL);
 
-	g_object_get (plugin, "shutdown", &daap_shutdown, NULL);
-	if (daap_shutdown == FALSE) {
+	if (rb_daap_plugin_shutdown (RB_DAAP_PLUGIN (plugin)) == FALSE) {
 
 		icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin),
 						source->priv->password_protected,
@@ -586,13 +613,16 @@ rb_daap_source_connection_cb (DMAPConnection   *connection,
 	for (l = playlists; l != NULL; l = g_slist_next (l)) {
 		DMAPPlaylist *playlist = l->data;
 		RBSource *playlist_source;
-		char *sorting_name;
+		char *settings_name;
 
-		/* Construct a unique sorting name for this playlist, as <Share Name>_<Playlist> */
-		sorting_name = g_strjoin (NULL, daap_source->priv->service_name, "_", playlist->name, NULL);
+		/* Construct a unique settings name for this playlist, as <Share Name>_<Playlist> */
+		/* XXX disabled for now, not sure it's a good idea */
+		/*settings_name = g_strjoin (NULL, daap_source->priv->service_name, "_", playlist->name, NULL); */
 
-		playlist_source = rb_static_playlist_source_new (shell, playlist->name, sorting_name, FALSE, entry_type);
-		g_free (sorting_name);
+		settings_name = "/org/gnome/rhythmbox/plugins/daap/source";
+
+		playlist_source = rb_static_playlist_source_new (shell, playlist->name, settings_name, FALSE, entry_type);
+		/*g_free (settings_name);*/
 
 		g_list_foreach (playlist->uris, (GFunc)_add_location_to_playlist, playlist_source);
 
@@ -629,10 +659,9 @@ rb_daap_source_selected (RBDisplayPage *page)
 
 	factory = DMAP_RECORD_FACTORY (rb_daap_record_factory_new ());
 
-	daap_source->priv->connection = dmap_connection_new (name,
+	daap_source->priv->connection = daap_connection_new (name,
 							     daap_source->priv->host,
 							     daap_source->priv->port,
-							     daap_source->priv->password_protected,
 							     db,
 							     factory);
 	g_object_unref (entry_type);
@@ -761,18 +790,6 @@ rb_daap_source_get_headers (RBDAAPSource *source,
 	return dmap_connection_get_headers (source->priv->connection, uri);
 }
 
-static char *
-rb_daap_source_get_browser_key (RBSource *source)
-{
-	return g_strdup (CONF_STATE_SHOW_BROWSER);
-}
-
-static char *
-rb_daap_source_get_paned_key (RBBrowserSource *source)
-{
-	return g_strdup (CONF_STATE_PANED_POSITION);
-}
-
 static void
 rb_daap_source_get_status (RBDisplayPage *page,
 			   char **text,
@@ -795,3 +812,10 @@ rb_daap_source_get_status (RBDisplayPage *page,
 
 	RB_DISPLAY_PAGE_CLASS (rb_daap_source_parent_class)->get_status (page, text, progress_text, progress);
 }
+
+void
+_rb_daap_source_register_type (GTypeModule *module)
+{
+	rb_daap_source_register_type (module);
+	rb_daap_entry_type_register_type (module);
+}
