@@ -37,27 +37,16 @@
 
 #include "mediaplayerid.h"
 
-#include "eel-gconf-extensions.h"
 #include "rb-nokia770-source.h"
 #include "rb-debug.h"
 #include "rb-util.h"
 #include "rb-file-helpers.h"
 #include "rhythmdb.h"
-#include "rb-plugin.h"
 
 
 static char * impl_uri_from_playlist_uri (RBGenericPlayerSource *source, const char *uri);
 
-
-typedef struct {
-#ifdef __SUNPRO_C
-   int x;  /* To build with Solaris forte compiler */
-#endif
-} RBNokia770SourcePrivate;
-
-RB_PLUGIN_DEFINE_TYPE (RBNokia770Source, rb_nokia770_source, RB_TYPE_GENERIC_PLAYER_SOURCE)
-#define NOKIA770_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_NOKIA770_SOURCE, RBNokia770SourcePrivate))
-
+G_DEFINE_DYNAMIC_TYPE (RBNokia770Source, rb_nokia770_source, RB_TYPE_GENERIC_PLAYER_SOURCE)
 
 #define NOKIA_INTERNAL_MOUNTPOINT "file:///media/mmc1/"
 
@@ -67,8 +56,11 @@ rb_nokia770_source_class_init (RBNokia770SourceClass *klass)
 	RBGenericPlayerSourceClass *generic_class = RB_GENERIC_PLAYER_SOURCE_CLASS (klass);
 
 	generic_class->impl_uri_from_playlist_uri = impl_uri_from_playlist_uri;
+}
 
-	g_type_class_add_private (klass, sizeof (RBNokia770SourcePrivate));
+static void
+rb_nokia770_source_class_finalize (RBNokia770SourceClass *klass)
+{
 }
 
 static void
@@ -78,10 +70,10 @@ rb_nokia770_source_init (RBNokia770Source *source)
 }
 
 RBRemovableMediaSource *
-rb_nokia770_source_new (RBPlugin *plugin, RBShell *shell, GMount *mount, MPIDDevice *device_info)
+rb_nokia770_source_new (GObject *plugin, RBShell *shell, GMount *mount, MPIDDevice *device_info)
 {
 	RBNokia770Source *source;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 	RhythmDB *db;
 	GVolume *volume;
 	char *name;
@@ -94,7 +86,14 @@ rb_nokia770_source_new (RBPlugin *plugin, RBShell *shell, GMount *mount, MPIDDev
 	g_object_get (G_OBJECT (shell), "db", &db, NULL);
 	path = g_volume_get_identifier (volume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	name = g_strdup_printf ("nokia770: %s", path);
-	entry_type = rhythmdb_entry_register_type (db, name);
+
+	entry_type = g_object_new (RHYTHMDB_TYPE_ENTRY_TYPE,
+				   "db", db,
+				   "name", name,
+				   "category", RHYTHMDB_ENTRY_NORMAL,
+				   "save-to-disk", FALSE,
+				   NULL);
+	rhythmdb_register_entry_type (db, entry_type);
 	g_object_unref (db);
 	g_free (name);
 	g_free (path);
@@ -103,11 +102,8 @@ rb_nokia770_source_new (RBPlugin *plugin, RBShell *shell, GMount *mount, MPIDDev
 	source = RB_NOKIA770_SOURCE (g_object_new (RB_TYPE_NOKIA770_SOURCE,
 						   "plugin", plugin,
 						   "entry-type", entry_type,
-						   "ignore-entry-type", RHYTHMDB_ENTRY_TYPE_INVALID,
-						   "error-entry-type", RHYTHMDB_ENTRY_TYPE_INVALID,
 						   "mount", mount,
 						   "shell", shell,
-						   "source-group", RB_SOURCE_GROUP_DEVICES,
 						   "device-info", device_info,
 						   NULL));
 
@@ -155,3 +151,8 @@ rb_nokia770_is_mount_player (GMount *mount, MPIDDevice *device_info)
 	return result;
 }
 
+void
+_rb_nokia770_source_register_type (GTypeModule *module)
+{
+	rb_nokia770_source_register_type (module);
+}
