@@ -58,10 +58,8 @@
 #include "rb-file-helpers.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
-#include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
 #include "rb-search-entry.h"
-#include "rb-preferences.h"
 #include "rb-shell-preferences.h"
 
 static void rb_browser_source_class_init (RBBrowserSourceClass *klass);
@@ -80,20 +78,11 @@ static void rb_browser_source_get_property (GObject *object,
 static void rb_browser_source_cmd_choose_genre (GtkAction *action, RBSource *source);
 static void rb_browser_source_cmd_choose_artist (GtkAction *action, RBSource *source);
 static void rb_browser_source_cmd_choose_album (GtkAction *action, RBSource *source);
-static void songs_view_sort_order_changed_cb (RBEntryView *view, RBBrowserSource *source);
+static void songs_view_sort_order_changed_cb (GObject *object, GParamSpec *pspec, RBBrowserSource *source);
 static void rb_browser_source_browser_changed_cb (RBLibraryBrowser *entry,
 						  GParamSpec *param,
 						  RBBrowserSource *source);
 
-static void rb_browser_source_state_prefs_sync (RBBrowserSource *source);
-static void rb_browser_source_state_pref_changed (GConfClient *client,
-							 guint cnxn_id,
-							 GConfEntry *entry,
-							 RBBrowserSource *source);
-
-static void paned_size_allocate_cb (GtkWidget *widget,
-				    GtkAllocation *allocation,
-		                    RBBrowserSource *source);
 /* source methods */
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static GList *impl_get_property_views (RBSource *source);
@@ -102,7 +91,6 @@ static void impl_search (RBSource *source, RBSourceSearch *search, const char *c
 static void impl_reset_filters (RBSource *source);
 static void impl_song_properties (RBSource *source);
 static GList *impl_get_search_actions (RBSource *source);
-static void impl_browser_toggled (RBSource *asource, gboolean disclosed);
 static void default_show_entry_popup (RBBrowserSource *source);
 static void default_pack_paned (RBBrowserSource *source, GtkWidget *paned);
 
@@ -138,11 +126,6 @@ struct RBBrowserSourcePrivate
 	GtkActionGroup *action_group;
 	GtkActionGroup *search_action_group;
 
-	char *sorting_key;
-	guint state_paned_notify_id;
-	guint state_browser_notify_id;
-	guint state_sorting_notify_id;
-
 	gboolean dispose_has_run;
 };
 
@@ -177,10 +160,10 @@ static const GtkTargetEntry songs_view_drag_types[] = {
 enum
 {
 	PROP_0,
-	PROP_SORTING_KEY,
 	PROP_BASE_QUERY_MODEL,
 	PROP_POPULATE,
-	PROP_SEARCH_TYPE
+	PROP_SEARCH_TYPE,
+	PROP_SHOW_BROWSER
 };
 
 G_DEFINE_ABSTRACT_TYPE (RBBrowserSource, rb_browser_source, RB_TYPE_SOURCE)
@@ -211,21 +194,11 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_delete = impl_delete;
 	source_class->impl_get_search_actions = impl_get_search_actions;
-	source_class->impl_browser_toggled = impl_browser_toggled;
 
 	klass->impl_pack_paned = default_pack_paned;
-	klass->impl_get_paned_key = (RBBrowserSourceStringFunc)rb_null_function;
 	klass->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 	klass->impl_show_entry_popup = default_show_entry_popup;
 
-	g_object_class_install_property (object_class,
-					 PROP_SORTING_KEY,
-					 g_param_spec_string ("sorting-key",
-							      "Sorting key",
-							      "GConf key for storing sort-order",
-							      NULL,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-
 	g_object_class_override_property (object_class,
 					  PROP_BASE_QUERY_MODEL,
 					  "base-query-model");
@@ -241,6 +214,9 @@ rb_browser_source_class_init (RBBrowserSourceClass *klass)
 	g_object_class_override_property (object_class,
 					  PROP_SEARCH_TYPE,
 					  "search-type");
+	g_object_class_override_property (object_class,
+					  PROP_SHOW_BROWSER,
+					  "show-browser");
 
 	g_type_class_add_private (klass, sizeof (RBBrowserSourcePrivate));
 }
@@ -289,10 +265,6 @@ rb_browser_source_dispose (GObject *object)
 		source->priv->default_search = NULL;
 	}
 
-	eel_gconf_notification_remove (source->priv->state_browser_notify_id);
-	eel_gconf_notification_remove (source->priv->state_paned_notify_id);
-	eel_gconf_notification_remove (source->priv->state_sorting_notify_id);
-
 	G_OBJECT_CLASS (rb_browser_source_parent_class)->dispose (object);
 }
 
@@ -308,8 +280,6 @@ rb_browser_source_finalize (GObject *object)
 
 	g_return_if_fail (source->priv != NULL);
 
-	g_free (source->priv->sorting_key);
-
 	G_OBJECT_CLASS (rb_browser_source_parent_class)->finalize (object);
 }
 
@@ -323,14 +293,14 @@ rb_browser_source_songs_show_popup_cb (RBEntryView *view,
 
 		klass->impl_show_entry_popup (source);
 	} else {
-		rb_source_show_popup (RB_SOURCE (source));
+		rb_display_page_show_popup (RB_DISPLAY_PAGE (source));
 	}
 }
 
 static void
 default_show_entry_popup (RBBrowserSource *source)
 {
-	_rb_source_show_popup (RB_SOURCE (source), "/BrowserSourceViewPopup");
+	_rb_display_page_show_popup (RB_DISPLAY_PAGE (source), "/BrowserSourceViewPopup");
 }
 
 static void
@@ -340,9 +310,7 @@ rb_browser_source_constructed (GObject *object)
 	RBBrowserSourceClass *klass;
 	RBShell *shell;
 	GObject *shell_player;
-	char *browser_key;
-	char *paned_key;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	RB_CHAIN_GOBJECT_METHOD (rb_browser_source_parent_class, constructed, object);
 
@@ -352,16 +320,18 @@ rb_browser_source_constructed (GObject *object)
 		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
-	g_object_get (shell, "db", &source->priv->db, NULL);
-	shell_player = rb_shell_get_player (shell);
+	g_object_get (shell,
+		      "db", &source->priv->db,
+		      "shell-player", &shell_player,
+		      NULL);
 
-	source->priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
-								       "BrowserSourceActions",
-								       NULL, 0, NULL);
-	_rb_action_group_add_source_actions (source->priv->action_group,
-					     G_OBJECT (shell),
-					     rb_browser_source_actions,
-					     G_N_ELEMENTS (rb_browser_source_actions));
+	source->priv->action_group = _rb_display_page_register_action_group (RB_DISPLAY_PAGE (source),
+									     "BrowserSourceActions",
+									     NULL, 0, NULL);
+	_rb_action_group_add_display_page_actions (source->priv->action_group,
+						   G_OBJECT (shell),
+						   rb_browser_source_actions,
+						   G_N_ELEMENTS (rb_browser_source_actions));
 
 	/* only add the actions if we haven't already */
 	if (gtk_action_group_get_action (source->priv->action_group,
@@ -384,6 +354,7 @@ rb_browser_source_constructed (GObject *object)
 	source->priv->paned = gtk_vpaned_new ();
 
 	source->priv->browser = rb_library_browser_new (source->priv->db, entry_type);
+	gtk_widget_set_no_show_all (GTK_WIDGET (source->priv->browser), TRUE);
 	gtk_paned_pack1 (GTK_PANED (source->priv->paned), GTK_WIDGET (source->priv->browser), TRUE, FALSE);
 	gtk_container_child_set (GTK_CONTAINER (source->priv->paned),
 				 GTK_WIDGET (source->priv->browser),
@@ -395,7 +366,6 @@ rb_browser_source_constructed (GObject *object)
 
 	/* set up songs tree view */
 	source->priv->songs = rb_entry_view_new (source->priv->db, shell_player,
-						 source->priv->sorting_key,
 						 TRUE, FALSE);
 
 	rb_entry_view_append_column (source->priv->songs, RB_ENTRY_VIEW_COL_TRACK_NUMBER, FALSE);
@@ -413,35 +383,15 @@ rb_browser_source_constructed (GObject *object)
 
 	g_signal_connect_object (G_OBJECT (source->priv->songs), "show_popup",
 				 G_CALLBACK (rb_browser_source_songs_show_popup_cb), source, 0);
-	g_signal_connect_object (G_OBJECT (source->priv->songs),
-				 "sort-order-changed",
+	g_signal_connect_object (source->priv->songs,
+				 "notify::sort-order",
 				 G_CALLBACK (songs_view_sort_order_changed_cb),
 				 source, 0);
 
-	if (source->priv->sorting_key) {
-		source->priv->state_sorting_notify_id =
-			eel_gconf_notification_add (source->priv->sorting_key,
-					    (GConfClientNotifyFunc) rb_browser_source_state_pref_changed,
-					    source);
-	}
-	paned_key = rb_browser_source_get_paned_key (source);
-	if (paned_key) {
-		source->priv->state_paned_notify_id =
-			eel_gconf_notification_add (paned_key,
-					    (GConfClientNotifyFunc) rb_browser_source_state_pref_changed,
-					    source);
-		g_free (paned_key);
-	}
-	browser_key = rb_source_get_browser_key (RB_SOURCE (source));
-	if (browser_key) {
-		source->priv->state_browser_notify_id =
-			eel_gconf_notification_add (browser_key,
-					    (GConfClientNotifyFunc) rb_browser_source_state_pref_changed,
-					    source);
-		g_free (browser_key);
-	}
-
-	rb_browser_source_state_prefs_sync (source);
+	rb_source_bind_settings (RB_SOURCE (source),
+				 GTK_WIDGET (source->priv->songs),
+				 source->priv->paned,
+				 GTK_WIDGET (source->priv->browser));
 
 	if (rb_browser_source_has_drop_support (source)) {
 		gtk_drag_dest_set (GTK_WIDGET (source->priv->songs),
@@ -460,12 +410,6 @@ rb_browser_source_constructed (GObject *object)
 					 source, 0);
 	}
 
-	/* this gets emitted when the paned thingie is moved */
-	g_signal_connect_object (G_OBJECT (source->priv->songs),
-				 "size_allocate",
-				 G_CALLBACK (paned_size_allocate_cb),
-				 source, 0);
-
 	gtk_paned_pack2 (GTK_PANED (source->priv->paned), GTK_WIDGET (source->priv->songs), TRUE, FALSE);
 
 	klass = RB_BROWSER_SOURCE_GET_CLASS (source);
@@ -481,7 +425,8 @@ rb_browser_source_constructed (GObject *object)
 	source->priv->cached_all_query = rhythmdb_query_model_new_empty (source->priv->db);
 	rb_browser_source_populate (source);
 
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
+	g_object_unref (shell_player);
 }
 
 static void
@@ -493,10 +438,6 @@ rb_browser_source_set_property (GObject *object,
 	RBBrowserSource *source = RB_BROWSER_SOURCE (object);
 
 	switch (prop_id) {
-	case PROP_SORTING_KEY:
-		g_free (source->priv->sorting_key);
-		source->priv->sorting_key = g_strdup (g_value_get_string (value));
-		break;
 	case PROP_POPULATE:
 		source->priv->populate = g_value_get_boolean (value);
 
@@ -508,6 +449,14 @@ rb_browser_source_set_property (GObject *object,
 	case PROP_SEARCH_TYPE:
 		/* ignored */
 		break;
+	case PROP_SHOW_BROWSER:
+		if (g_value_get_boolean (value)) {
+			gtk_widget_show (GTK_WIDGET (source->priv->browser));
+		} else {
+			gtk_widget_hide (GTK_WIDGET (source->priv->browser));
+			rb_library_browser_reset (source->priv->browser);
+		}
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -523,9 +472,6 @@ rb_browser_source_get_property (GObject *object,
 	RBBrowserSource *source = RB_BROWSER_SOURCE (object);
 
 	switch (prop_id) {
-	case PROP_SORTING_KEY:
-		g_value_set_string (value, source->priv->sorting_key);
-		break;
 	case PROP_BASE_QUERY_MODEL:
 		g_value_set_object (value, source->priv->cached_all_query);
 		break;
@@ -535,6 +481,9 @@ rb_browser_source_get_property (GObject *object,
 	case PROP_SEARCH_TYPE:
 		g_value_set_enum (value, RB_SOURCE_SEARCH_INCREMENTAL);
 		break;
+	case PROP_SHOW_BROWSER:
+		g_value_set_boolean (value, gtk_widget_get_visible (GTK_WIDGET (source->priv->browser)));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -570,7 +519,7 @@ rb_browser_source_populate (RBBrowserSource *source)
 				      RHYTHMDB_QUERY_RESULTS (source->priv->cached_all_query),
 				      RHYTHMDB_QUERY_PROP_EQUALS, RHYTHMDB_PROP_TYPE, entry_type,
 				      RHYTHMDB_QUERY_END);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 }
 
 static void
@@ -619,10 +568,10 @@ rb_browser_source_cmd_choose_album (GtkAction *action, RBSource *source)
 }
 
 static void
-songs_view_sort_order_changed_cb (RBEntryView *view, RBBrowserSource *source)
+songs_view_sort_order_changed_cb (GObject *object, GParamSpec *pspec, RBBrowserSource *source)
 {
 	rb_debug ("sort order changed");
-	rb_entry_view_resort_model (view);
+	rb_entry_view_resort_model (RB_ENTRY_VIEW (object));
 }
 
 static void
@@ -727,53 +676,6 @@ impl_song_properties (RBSource *asource)
 		rb_debug ("failed to create dialog, or no selection!");
 }
 
-static void
-paned_size_allocate_cb (GtkWidget *widget,
-			GtkAllocation *allocation,
-		        RBBrowserSource *source)
-{
-	char *key = rb_browser_source_get_paned_key (source);;
-
-	/* save state */
-	rb_debug ("paned size allocate");
-	if (key)
-		eel_gconf_set_integer (key, gtk_paned_get_position (GTK_PANED (source->priv->paned)));
-	g_free (key);
-}
-
-static void
-rb_browser_source_state_pref_changed (GConfClient *client,
-				     guint cnxn_id,
-				     GConfEntry *entry,
-				     RBBrowserSource *source)
-{
-	rb_debug ("state prefs changed");
-	rb_browser_source_state_prefs_sync (source);
-}
-
-static void
-rb_browser_source_state_prefs_sync (RBBrowserSource *source)
-{
-	char *paned_key;
-	char *browser_key;
-
-	rb_debug ("syncing state");
-	paned_key = rb_browser_source_get_paned_key (source);
-	if (paned_key)
-		gtk_paned_set_position (GTK_PANED (source->priv->paned),
-					eel_gconf_get_integer (paned_key));
-
-	browser_key = rb_source_get_browser_key (RB_SOURCE (source));
-	if (browser_key && eel_gconf_get_boolean (browser_key)) {
-		gtk_widget_show (GTK_WIDGET (source->priv->browser));
-	} else {
-		gtk_widget_hide (GTK_WIDGET (source->priv->browser));
-	}
-
-	g_free (paned_key);
-	g_free (browser_key);
-}
-
 static GList *
 impl_get_search_actions (RBSource *source)
 {
@@ -787,42 +689,9 @@ impl_get_search_actions (RBSource *source)
 	return actions;
 }
 
-static void
-impl_browser_toggled (RBSource *asource, gboolean disclosed)
-{
-	RBBrowserSource *source = RB_BROWSER_SOURCE (asource);
-
-	if (disclosed) {
-		gtk_widget_show (GTK_WIDGET (source->priv->browser));
-	} else {
-		gtk_widget_hide (GTK_WIDGET (source->priv->browser));
-		rb_library_browser_reset (source->priv->browser);
-	}
-}
-
-/**
- * rb_browser_source_get_paned_key:
- * @source: a #RBBrowserSource
- *
- * Retrieves the GConf key that stores the height of the browser pane for the source.
- * This is a virtual method that should be implemented by subclasses.
- *
- * Return value: allocated string containing the GConf key name
- */
-char *
-rb_browser_source_get_paned_key (RBBrowserSource *source)
-{
-	RBBrowserSourceClass *klass = RB_BROWSER_SOURCE_GET_CLASS (source);
-
-	if (klass->impl_get_paned_key)
-		return klass->impl_get_paned_key (source);
-	else
-		return NULL;
-}
-
 /**
  * rb_browser_source_has_drop_support:
- * @source: a #RBBrowser
+ * @source: a #RBBrowserSource
  *
  * This is a virtual method that should be implemented by subclasses.  It returns %TRUE
  * if drag and drop target support for the source should be activated.
@@ -846,7 +715,7 @@ songs_view_drag_data_received_cb (GtkWidget *widget,
 				  RBBrowserSource *source)
 {
 	rb_debug ("data dropped on the library source song view");
-	rb_source_receive_drag (RB_SOURCE (source), selection_data);
+	rb_display_page_receive_drag (RB_DISPLAY_PAGE (source), selection_data);
 }
 
 static void
@@ -884,7 +753,7 @@ rb_browser_source_do_query (RBBrowserSource *source, gboolean subset)
 {
 	RhythmDBQueryModel *query_model;
 	GPtrArray *query;
-	RhythmDBEntryType entry_type;
+	RhythmDBEntryType *entry_type;
 
 	/* use the cached 'all' query to optimise the no-search case */
 	if (source->priv->search_query == NULL) {
@@ -902,7 +771,7 @@ rb_browser_source_do_query (RBBrowserSource *source, gboolean subset)
 				      RHYTHMDB_QUERY_SUBQUERY,
 				      source->priv->search_query,
 				      RHYTHMDB_QUERY_END);
-	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
+	g_object_unref (entry_type);
 
 	if (subset) {
 		/* if we're appending text to an existing search string, the results will be a subset
@@ -948,4 +817,3 @@ default_pack_paned (RBBrowserSource *source, GtkWidget *paned)
 	gtk_box_pack_start (GTK_BOX (box), paned, TRUE, TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (source), box);
 }
-
