@@ -67,12 +67,13 @@ static void rb_missing_files_source_get_property (GObject *object,
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static void impl_song_properties (RBSource *source);
 static void impl_delete (RBSource *source);
-static void impl_get_status (RBSource *source, char **text, char **progress_text, float *progress);
+static void impl_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress);
 
 static void rb_missing_files_source_songs_show_popup_cb (RBEntryView *view,
 							 gboolean over_entry,
 							 RBMissingFilesSource *source);
-static void rb_missing_files_source_songs_sort_order_changed_cb (RBEntryView *view,
+static void rb_missing_files_source_songs_sort_order_changed_cb (GObject *object,
+								 GParamSpec *pspec,
 								 RBMissingFilesSource *source);
 
 #define MISSING_FILES_SOURCE_SONGS_POPUP_PATH "/MissingFilesViewPopup"
@@ -89,6 +90,7 @@ static void
 rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	RBDisplayPageClass *page_class = RB_DISPLAY_PAGE_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 
 	object_class->dispose = rb_missing_files_source_dispose;
@@ -97,6 +99,8 @@ rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 	object_class->set_property = rb_missing_files_source_set_property;
 	object_class->get_property = rb_missing_files_source_get_property;
 
+	page_class->get_status = impl_get_status;
+
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_get_entry_view = impl_get_entry_view;
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_false_function;
@@ -113,8 +117,6 @@ rb_missing_files_source_class_init (RBMissingFilesSourceClass *klass)
 	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_pause = (RBSourceFeatureFunc) rb_false_function;
 
-	source_class->impl_get_status = impl_get_status;
-
 	g_type_class_add_private (klass, sizeof (RBMissingFilesSourcePrivate));
 }
 
@@ -131,7 +133,7 @@ rb_missing_files_source_init (RBMissingFilesSource *source)
 					   "dialog-warning",
 					   size,
 					   0, NULL);
-	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
+	g_object_set (source, "pixbuf", pixbuf, NULL);
 	if (pixbuf != NULL) {
 		g_object_unref (pixbuf);
 	}
@@ -154,8 +156,10 @@ rb_missing_files_source_constructed (GObject *object)
 		      "shell", &shell,
 		      "entry-type", &entry_type,
 		      NULL);
-	g_object_get (shell, "db", &source->priv->db, NULL);
-	shell_player = rb_shell_get_player (shell);
+	g_object_get (shell,
+		      "db", &source->priv->db,
+		      "shell-player", &shell_player,
+		      NULL);
 	g_object_unref (shell);
 
 	/* construct real query */
@@ -178,7 +182,8 @@ rb_missing_files_source_constructed (GObject *object)
 
 	/* set up entry view */
 	source->priv->view = rb_entry_view_new (source->priv->db, shell_player,
-						NULL, FALSE, FALSE);
+						FALSE, FALSE);
+	g_object_unref (shell_player);
 
 	rb_entry_view_set_model (source->priv->view, model);
 
@@ -195,7 +200,7 @@ rb_missing_files_source_constructed (GObject *object)
 	gtk_container_add (GTK_CONTAINER (source), GTK_WIDGET (source->priv->view));
 	g_signal_connect_object (source->priv->view, "show_popup",
 				 G_CALLBACK (rb_missing_files_source_songs_show_popup_cb), source, 0);
-	g_signal_connect_object (source->priv->view, "sort-order-changed",
+	g_signal_connect_object (source->priv->view, "notify::sort-order",
 				 G_CALLBACK (rb_missing_files_source_songs_sort_order_changed_cb), source, 0);
 
 	gtk_widget_show_all (GTK_WIDGET (source));
@@ -283,7 +288,6 @@ rb_missing_files_source_new (RBShell *shell,
 					  "shell", shell,
 					  "visibility", FALSE,
 					  "hidden-when-empty", TRUE,
-					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
 	g_object_unref (entry_type);
 	return source;
@@ -295,7 +299,7 @@ rb_missing_files_source_songs_show_popup_cb (RBEntryView *view,
 					     RBMissingFilesSource *source)
 {
 	if (over_entry)
-		_rb_source_show_popup (RB_SOURCE (source), MISSING_FILES_SOURCE_SONGS_POPUP_PATH);
+		_rb_display_page_show_popup (RB_DISPLAY_PAGE (source), MISSING_FILES_SOURCE_SONGS_POPUP_PATH);
 }
 
 static void
@@ -330,19 +334,20 @@ impl_delete (RBSource *asource)
 }
 
 static void
-rb_missing_files_source_songs_sort_order_changed_cb (RBEntryView *view,
+rb_missing_files_source_songs_sort_order_changed_cb (GObject *object,
+						     GParamSpec *pspec,
 						     RBMissingFilesSource *source)
 {
-	rb_entry_view_resort_model (view);
+	rb_entry_view_resort_model (RB_ENTRY_VIEW (object));
 }
 
 static void
-impl_get_status (RBSource *asource, char **text, char **progress_text, float *progress)
+impl_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress)
 {
 	RhythmDBQueryModel *model;
 	gint count;
 
-	g_object_get (asource, "query-model", &model, NULL);
+	g_object_get (page, "query-model", &model, NULL);
 	count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);
 	g_object_unref (model);
 
