@@ -76,9 +76,8 @@ static GList * impl_cut (RBSource *source);
 static RBTrackTransferBatch *impl_paste (RBSource *asource, GList *entries);
 static void impl_delete (RBSource *source);
 static void impl_search (RBSource *asource, RBSourceSearch *search, const char *cur_text, const char *new_text);
-static void impl_browser_toggled (RBSource *source, gboolean enabled);
 static void impl_reset_filters (RBSource *asource);
-static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
+static gboolean impl_receive_drag (RBDisplayPage *page, GtkSelectionData *data);
 static GList *impl_get_search_actions (RBSource *source);
 static guint impl_want_uri (RBSource *source, const char *uri);
 
@@ -130,7 +129,8 @@ static GtkRadioActionEntry rb_static_playlist_source_radio_actions [] =
 enum
 {
 	PROP_0,
-	PROP_BASE_QUERY_MODEL
+	PROP_BASE_QUERY_MODEL,
+	PROP_SHOW_BROWSER
 };
 
 G_DEFINE_TYPE (RBStaticPlaylistSource, rb_static_playlist_source, RB_TYPE_PLAYLIST_SOURCE)
@@ -160,6 +160,7 @@ static void
 rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	RBDisplayPageClass *page_class = RB_DISPLAY_PAGE_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
 
@@ -169,17 +170,17 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	object_class->set_property = rb_static_playlist_source_set_property;
 	object_class->get_property = rb_static_playlist_source_get_property;
 
+	page_class->receive_drag = impl_receive_drag;
+
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_cut = impl_cut;
 	source_class->impl_paste = impl_paste;
 	source_class->impl_delete = impl_delete;
-	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_search = impl_search;
 	source_class->impl_reset_filters = impl_reset_filters;
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
-	source_class->impl_browser_toggled = impl_browser_toggled;
 	source_class->impl_get_property_views = impl_get_property_views;
 	source_class->impl_get_search_actions = impl_get_search_actions;
 	source_class->impl_want_uri = impl_want_uri;
@@ -189,6 +190,9 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	g_object_class_override_property (object_class,
 					  PROP_BASE_QUERY_MODEL,
 					  "base-query-model");
+	g_object_class_override_property (object_class,
+					  PROP_SHOW_BROWSER,
+					  "show-browser");
 
 	g_type_class_add_private (klass, sizeof (RBStaticPlaylistSourcePrivate));
 }
@@ -207,13 +211,13 @@ rb_static_playlist_source_init (RBStaticPlaylistSource *source)
 			g_object_add_weak_pointer (playlist_pixbuf,
 					   (gpointer *) &playlist_pixbuf);
 
-			rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+			g_object_set (source, "pixbuf", playlist_pixbuf, NULL);
 
 			/* drop the initial reference to the icon */
 			g_object_unref (playlist_pixbuf);
 		}
 	} else {
-		rb_source_set_pixbuf (RB_SOURCE (source), playlist_pixbuf);
+		g_object_set (source, "pixbuf", playlist_pixbuf, NULL);
 	}
 }
 
@@ -297,10 +301,10 @@ rb_static_playlist_source_constructed (GObject *object)
 	priv->paned = gtk_vpaned_new ();
 
 	g_object_get (source, "shell", &shell, NULL);
-	priv->action_group = _rb_source_register_action_group (RB_SOURCE (source),
-							       "StaticPlaylistActions",
-							       NULL, 0,
-							       shell);
+	priv->action_group = _rb_display_page_register_action_group (RB_DISPLAY_PAGE (source),
+								     "StaticPlaylistActions",
+								     NULL, 0,
+								     shell);
 	if (gtk_action_group_get_action (priv->action_group,
 					 rb_static_playlist_source_radio_actions[0].name) == NULL) {
 		gtk_action_group_add_radio_actions (priv->action_group,
@@ -338,6 +342,8 @@ rb_static_playlist_source_constructed (GObject *object)
 	gtk_container_remove (GTK_CONTAINER (source), GTK_WIDGET (songs));
 	gtk_paned_pack2 (GTK_PANED (priv->paned), GTK_WIDGET (songs), TRUE, FALSE);
 	gtk_container_add (GTK_CONTAINER (source), priv->paned);
+
+	rb_source_bind_settings (RB_SOURCE (source), GTK_WIDGET (songs), priv->paned, GTK_WIDGET (priv->browser));
 	g_object_unref (songs);
 
 	/* watch these to find out when things are dropped into the entry view */
@@ -358,7 +364,7 @@ rb_static_playlist_source_constructed (GObject *object)
  * rb_static_playlist_source_new:
  * @shell: the #RBShell
  * @name: the playlist name
- * @sorting_name: the sorting name for the playlist (GConf key friendly)
+ * @settings_name: the settings name for the playlist (GSettings path friendly)
  * @local: if %TRUE, the playlist is local to the library
  * @entry_type: type of database entries that can be added to the playlist.
  *
@@ -367,21 +373,28 @@ rb_static_playlist_source_constructed (GObject *object)
  * Return value: new playlist.
  */
 RBSource *
-rb_static_playlist_source_new (RBShell *shell, const char *name, const char *sorting_name, gboolean local, RhythmDBEntryType *entry_type)
+rb_static_playlist_source_new (RBShell *shell, const char *name, const char *settings_name, gboolean local, RhythmDBEntryType *entry_type)
 {
+	GSettings *settings;
+
 	if (name == NULL)
 		name = "";
 
-	if (sorting_name == NULL)
-		sorting_name = "";
+	if (settings_name != NULL) {
+		char *path;
+		path = g_strdup_printf ("/org/gnome/rhythmbox/playlist/%s/", settings_name);
+		settings = g_settings_new_with_path ("org.gnome.rhythmbox.source", path);
+		g_free (path);
+	} else {
+		settings = NULL;
+	}
 
 	return RB_SOURCE (g_object_new (RB_TYPE_STATIC_PLAYLIST_SOURCE,
 					"name", name,
-					"sorting-name", sorting_name,
+					"settings", settings,
 					"shell", shell,
 					"is-local", local,
 					"entry-type", entry_type,
-					"source-group", RB_SOURCE_GROUP_PLAYLISTS,
 					"search-type", RB_SOURCE_SEARCH_INCREMENTAL,
 					NULL));
 }
@@ -392,9 +405,15 @@ rb_static_playlist_source_set_property (GObject *object,
 					const GValue *value,
 					GParamSpec *pspec)
 {
-	/*RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);*/
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);
 
 	switch (prop_id) {
+	case PROP_SHOW_BROWSER:
+		if (g_value_get_boolean (value))
+			gtk_widget_show (GTK_WIDGET (priv->browser));
+		else
+			gtk_widget_hide (GTK_WIDGET (priv->browser));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -413,6 +432,9 @@ rb_static_playlist_source_get_property (GObject *object,
 	case PROP_BASE_QUERY_MODEL:
 		g_value_set_object (value, priv->base_model);
 		break;
+	case PROP_SHOW_BROWSER:
+		g_value_set_boolean (value, gtk_widget_get_visible (GTK_WIDGET (priv->browser)));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -563,19 +585,6 @@ impl_get_property_views (RBSource *source)
 	return ret;
 }
 
-static void
-impl_browser_toggled (RBSource *source, gboolean enabled)
-{
-	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
-
-	priv->browser_shown = enabled;
-
-	if (enabled)
-		gtk_widget_show (GTK_WIDGET (priv->browser));
-	else
-		gtk_widget_hide (GTK_WIDGET (priv->browser));
-}
-
 static GPtrArray *
 construct_query_from_selection (RBStaticPlaylistSource *source)
 {
@@ -635,11 +644,11 @@ rb_static_playlist_source_browser_changed_cb (RBLibraryBrowser *browser,
 }
 
 static gboolean
-impl_receive_drag (RBSource *asource, GtkSelectionData *data)
+impl_receive_drag (RBDisplayPage *page, GtkSelectionData *data)
 {
 	GdkAtom type;
 	GList *list;
-	RBStaticPlaylistSource *source = RB_STATIC_PLAYLIST_SOURCE (asource);
+	RBStaticPlaylistSource *source = RB_STATIC_PLAYLIST_SOURCE (page);
 
 	type = gtk_selection_data_get_data_type (data);
 
@@ -790,7 +799,7 @@ _add_location_cb (GFile *file,
 		rb_static_playlist_source_add_location_internal (source, uri, -1);
 		g_free (uri);
 	}
-	return TRUE;	
+	return TRUE;
 }
 
 /**
@@ -828,7 +837,7 @@ rb_static_playlist_source_add_location (RBStaticPlaylistSource *source,
 /**
  * rb_static_playlist_source_add_locations:
  * @source: an #RBStaticPlaylistSource
- * @locations: a #GList of strings to add
+ * @locations: (element-type utf8) (transfer none): URI strings to add
  *
  * Adds the locations specified in @locations to the playlist.
  * See @rb_static_playlist_source_add_location for details.
