@@ -38,7 +38,6 @@
 #include "rb-query-creator-private.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
-#include "rb-preferences.h"
 #include "rb-builder-helpers.h"
 #include "rb-util.h"
 
@@ -216,7 +215,6 @@ rb_query_creator_constructed (GObject *object)
 
 	gtk_container_set_border_width (GTK_CONTAINER (creator), 5);
 	gtk_box_set_spacing (GTK_BOX (content_area), 2);
-	gtk_dialog_set_has_separator (GTK_DIALOG (creator), FALSE);
 
 	builder = rb_builder_load ("create-playlist.ui", creator);
 
@@ -522,7 +520,7 @@ rb_query_creator_new_from_query (RhythmDB *db,
  *
  * Extracts a child widget from a #GtkBox.
  *
- * Return value: child widget from the box
+ * Return value: (transfer none): child widget from the box
  */
 GtkWidget *
 get_box_widget_at_pos (GtkBox *box, guint pos)
@@ -566,7 +564,7 @@ get_entry_for_property (RBQueryCreator *creator,
  *
  * Constructs a database query that represents the criteria in the query creator.
  *
- * Return value: database query array
+ * Return value: (transfer full): database query array
  */
 GPtrArray *
 rb_query_creator_get_query (RBQueryCreator *creator)
@@ -630,8 +628,8 @@ rb_query_creator_get_query (RBQueryCreator *creator)
 /**
  * rb_query_creator_get_limit:
  * @creator: #RBQueryCreator instance
- * @type: used to return the limit type
- * @limit: used to return the limit value
+ * @type: (out): used to return the limit type
+ * @limit: (out): used to return the limit value
  *
  * Retrieves the limit type and value from the query creator.
  * The limit value is returned as the first element in a
@@ -686,8 +684,8 @@ rb_query_creator_get_limit (RBQueryCreator *creator,
 /**
  * rb_query_creator_get_sort_order:
  * @creator: #RBQueryCreator instance
- * @sort_key: returns the sort key name
- * @sort_direction: returns the sort direction
+ * @sort_key: (out) (allow-none): returns the sort key name
+ * @sort_direction: (out) (allow-none): returns the sort direction
  *
  * Retrieves the sort settings from the query creator.
  * The sort direction is returned as a #GtkSortType value.
@@ -783,13 +781,8 @@ append_row (RBQueryCreator *creator)
 	GtkWidget *entry;
 	GtkWidget *remove_button;
 	GtkBox *hbox;
-	GList *rows;
-	guint len;
 	gboolean constrain;
 
-	rows = priv->rows;
-	len = g_list_length (rows);
-
 	hbox = GTK_BOX (gtk_hbox_new (FALSE, 5));
 	gtk_box_pack_start (GTK_BOX (priv->vbox), GTK_WIDGET (hbox), TRUE, TRUE, 0);
 	priv->rows = g_list_prepend (priv->rows, hbox);
@@ -915,9 +908,9 @@ create_property_option_menu (RBQueryCreator *creator,
 	GtkWidget *combo;
 	int i;
 
-	combo = gtk_combo_box_new_text ();
+	combo = gtk_combo_box_text_new ();
 	for (i = 0; i < length; i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), g_dpgettext2 (NULL, "query-criteria", options[i].name));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), g_dpgettext2 (NULL, "query-criteria", options[i].name));
 	}
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
@@ -937,9 +930,9 @@ create_criteria_option_menu (const RBQueryCreatorCriteriaOption *options,
 	GtkWidget *combo;
 	int i;
 
-	combo = gtk_combo_box_new_text ();
+	combo = gtk_combo_box_text_new ();
 	for (i = 0; i < length; i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(options[i].name));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _(options[i].name));
 	}
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
